/* triples: Find primitive Pythagorean triples. */
/* Original date: 2014-01-25 */
/* Version: 2.0 */
/* Date: 2024-07-12 */

#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <math.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

/* Get more prime numbers. */
static unsigned long int
more_primes (size_t *const sizep, unsigned long int **const primesp,
	     const size_t new_size)
{
  unsigned long int *const new =
    realloc (*primesp, new_size * sizeof (**primesp));
  if (new == NULL)
    {
      return false;
    }
  else
    {
      unsigned long int *const primes = *primesp = new;
      unsigned long int c = primes[*sizep - 1] + 2;
      unsigned short int sqrt_c = sqrt (c);
      unsigned long int next_square = 1lu * (sqrt_c + 1) * (sqrt_c + 1);
      do
	{
	  size_t j = 0;
	  while (primes[j] <= sqrt_c)
	    {
	      if ((c % primes[j]) == 0)
		{
		  c += 2;
		  if (c >= next_square)
		    {
		      sqrt_c++;
		      next_square = 1lu * (sqrt_c + 1) * (sqrt_c + 1);
		    }
		  j = 0;
		}
	      else
		{
		  j++;
		}
	    }
	  primes[(*sizep)++] = c;
	  c += 2;
	  if (c >= next_square)
	    {
	      sqrt_c++;
	      next_square = 1lu * (sqrt_c + 1) * (sqrt_c + 1);
	    }
	}
      while (*sizep < new_size);
      return true;
    }
}

/* Check whether two integers are coprime. */
static bool
coprime (unsigned long int i, unsigned long int j)
{
  while (i > 1 && j > 1)
    {
      if (i > j)
	{
	  i %= j;
	}
      else
	{
	  j %= i;
	}
    }
  if (i == 0 || j == 0)
    {
      return false;
    }
  else
    {
      return true;
    }
}

/* If the arguments can form a primitive Pythagorean triple, return the second integer to generate the triple. Otherwise, return 0. */
static unsigned long int
find_second_integer (const unsigned long int i,
		     const unsigned long long int c)
{
  unsigned long long int x = c - 1llu * i * i;
  const double f = sqrt (x);
  const double g = trunc (f);
  if (f != g || coprime (i, g) == false)
    {
      return 0;
    }
  else
    {
      return g;
    }
}

/* Sort triples. */
static void
sort_triples (const size_t count,
	      unsigned long long int *const restrict triples)
{
  size_t g;
  for (g = count / 2; g > 0; g /= 2)
    {
      const size_t h = count - g;
      size_t i;
      for (i = 0; i < g; i++)
	{
	  size_t j = i;
	  do
	    {
	      const size_t k = j + g;
	      size_t l = 3 * j;
	      size_t m = 3 * k;
	      if (triples[l] > triples[m])
		{
		  unsigned long long int temp = triples[l];
		  triples[l] = triples[m];
		  triples[m] = temp;
		  l++;
		  m++;
		  temp = triples[l];
		  triples[l] = triples[m];
		  triples[m] = temp;
		  l++;
		  m++;
		  if (triples[l] != triples[m])
		    {
		      temp = triples[l];
		      triples[l] = triples[m];
		      triples[m] = temp;
		    }
		  if (j != i)
		    {
		      j -= g;
		    }
		  else
		    {
		      j = k;
		    }
		}
	      else
		{
		  j = k;
		}
	    }
	  while (j < h);
	}
    }
}

/* Print triples. */
static void
print_triples (const size_t count,
	       const unsigned long long int *const restrict triples,
	       const char *const restrict format)
{
  size_t i;
  for (i = 0; i < count; i++)
    {
      const size_t j = 3 * i;
      printf (format, triples[j], triples[j + 1], triples[j + 2]);
    }
}

/* Comparison function to use with qsort. */
static int
cmplu (const void *const restrict ap, const void *const restrict bp)
{
  unsigned long int a = *(const unsigned long long int *) ap;
  unsigned long int b = *(const unsigned long long int *) bp;
  if (a > b)
    {
      return 1;
    }
  else if (a == b)
    {
      return 0;
    }
  else
    {
      return -1;
    }
}

/* Find and print primitive Pythagorean triples that include number. */
static bool
find_triples_include (const char *const restrict program_name,
		      const size_t line, const unsigned long long int number,
		      const unsigned long long int max_even,
		      const unsigned long long int max_odd,
		      size_t *restrict primes_sizep,
		      unsigned long int **const restrict primesp,
		      const char *const restrict format)
{
  size_t primes_size = *primes_sizep;
  unsigned long int *primes = *primesp;
  static size_t ppf_size = 0;
  static unsigned long int *ppf = NULL;
  static size_t sf_size = 0;
  static unsigned long int *sf = NULL;
  if ((number % 2) == 1)
    {
      if (number > max_odd)
	{
	  fprintf (stderr, "%s: ", program_name);
	  if (line != 0)
	    {
	      fprintf (stderr, "%zu: ", line);
	    }
	  fprintf (stderr,
		   "%llu: integer gives triple with integer overflow\n",
		   number);
	  return false;
	}
      else if (number != 1)
	{
	  static size_t triples_size = 0;
	  static unsigned long long int *triples = NULL;
	  size_t count = 0;
	  size_t count_c = 0;
	  bool sort = false;
	  if ((number % 4) == 1)
	    {
	      const unsigned long int start =
		(sqrt (2 * (number - 1) + 1) + 1) / 2;
	      const unsigned long int end = sqrt (number - 1);
	      const unsigned long int boundary =
		sqrt ((3 + 2 * sqrt (2)) * number / (4 + 2 * sqrt (2)));
	      unsigned long int i;
	      for (i = start; i <= boundary; i++)
		{
		  const unsigned long int j = find_second_integer (i, number);
		  if (j != 0)
		    {
		      if (count == triples_size)
			{
			  const size_t new_size = triples_size + 1;
			  unsigned long long int *const new =
			    realloc (triples,
				     new_size * 3 * sizeof (*triples));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      triples_size = new_size;
			      triples = new;
			    }
			}
		      size_t index = count * 3;
		      triples[index++] = 1llu * i * i - 1llu * j * j;
		      triples[index++] = 2llu * i * j;
		      triples[index] = number;
		      count++;
		    }
		}
	      count_c = count;
	      for (i = end; i > boundary; i--)
		{
		  const unsigned long int j = find_second_integer (i, number);
		  if (j != 0)
		    {
		      if (count == triples_size)
			{
			  const size_t new_size = triples_size + 1;
			  unsigned long long int *const new =
			    realloc (triples,
				     new_size * 3 * sizeof (*triples));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      triples_size = new_size;
			      triples = new;
			    }
			}
		      size_t index = count * 3;
		      triples[index++] = 2llu * i * j;
		      triples[index++] = 1llu * i * i - 1llu * j * j;
		      triples[index] = number;
		      count++;
		    }
		}
	      if (count_c != 0 && count_c != count)
		{
		  sort = true;
		}
	      count_c = count;
	    }
	  unsigned long long int x = number;
	  unsigned long int boundary = sqrt (x);
	  unsigned long int sqrt_x = boundary;
	  size_t n = 0;
	  size_t i = 1;
	  unsigned long int p = primes[i++];
	  size_t t;
	  size_t k;
	  size_t m = 0;
	  while (x > ULONG_MAX && p <= sqrt_x)
	    {
	      if ((x % p) == 0)
		{
		  unsigned long long int f = 1;
		  do
		    {
		      f *= p;
		      x /= p;
		    }
		  while ((x % p) == 0);
		  sqrt_x = sqrt (x);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  unsigned long int z = x;
	  while (p <= sqrt_x)
	    {
	      if ((z % p) == 0)
		{
		  unsigned long int f = 1;
		  do
		    {
		      f *= p;
		      z /= p;
		    }
		  while ((z % p) == 0);
		  sqrt_x = sqrt (z);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  if (z != 1)
	    {
	      if (z <= boundary)
		{
		  if (n == ppf_size)
		    {
		      const size_t new_size = ppf_size + 1;
		      unsigned long int *const new =
			realloc (ppf, new_size * sizeof (*ppf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  ppf_size = new_size;
			  ppf = new;
			}
		    }
		  ppf[n++] = z;
		}
	    }
	  if (n > 1)
	    {
	      qsort (ppf, n, sizeof (*ppf), cmplu);
	    }
	  t = 1 << n;
	  for (k = 0; k < t; k++)
	    {
	      size_t j;
	      unsigned long long int f = 1;
	      for (i = 0, j = 1; i < n; i++, j <<= 1)
		{
		  if ((k & j) != 0)
		    {
		      f *= ppf[i];
		    }
		}
	      if (f <= boundary)
		{
		  if (m == sf_size)
		    {
		      const size_t new_size = sf_size + 1;
		      unsigned long int *const new =
			realloc (sf, new_size * sizeof (*sf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  sf_size = new_size;
			  sf = new;
			}
		    }
		  sf[m++] = f;
		}
	    }
	  if (m > 1)
	    {
	      qsort (sf, m, sizeof (*sf), cmplu);
	    }
	  if (count != 0)
	    {
	      for (i = 0; i < m; i++)
		{
		  const unsigned long long int f = sf[m - 1 - i];
		  const unsigned long long int g = number / f;
		  const unsigned long long int u = (g + f) / 2;
		  const unsigned long long int v = (g - f) / 2;
		  const unsigned long long int b = 2 * u * v;
		  if (b > number)
		    {
		      break;
		    }
		  else
		    {
		      if (count == triples_size)
			{
			  const size_t new_size = triples_size + 1;
			  unsigned long long int *const new =
			    realloc (triples,
				     new_size * 3 * sizeof (*triples));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      triples_size = new_size;
			      triples = new;
			    }
			}
		      size_t index = count * 3;
		      triples[index++] = b;
		      triples[index++] = number;
		      triples[index] = u * u + v * v;
		      count++;
		    }
		}
	      if (sort == false && count != count_c)
		{
		  sort = true;
		}
	      if (sort == true)
		{
		  sort_triples (count, triples);
		}
	      print_triples (count, triples, format);
	    }
	  else
	    {
	      for (i = 0; i < m; i++)
		{
		  const unsigned long long int f = sf[m - 1 - i];
		  const unsigned long long int g = number / f;
		  const unsigned long long int u = (g + f) / 2;
		  const unsigned long long int v = (g - f) / 2;
		  const unsigned long long int b = 2 * u * v;
		  if (b > number)
		    {
		      break;
		    }
		  else
		    {
		      const unsigned long long int c = u * u + v * v;
		      printf (format, b, number, c);
		    }
		}
	    }
	  for (; i < m; i++)
	    {
	      const unsigned long long int f = sf[m - 1 - i];
	      const unsigned long long int g = number / f;
	      const unsigned long long int u = (g + f) / 2;
	      const unsigned long long int v = (g - f) / 2;
	      const unsigned long long int b = 2 * u * v;
	      const unsigned long long int c = u * u + v * v;
	      printf (format, number, b, c);
	    }
	}
    }
  else if ((number % 4) == 0)
    {
      if (number >= max_even)
	{
	  fprintf (stderr, "%s: ", program_name);
	  if (line != 0)
	    {
	      fprintf (stderr, "%zu: ", line);
	    }
	  fprintf (stderr,
		   "%llu: integer gives triple with integer overflow\n",
		   number);
	  return false;
	}
      else
	{
	  unsigned long long int x = number / 2;
	  const unsigned long long int y = x;
	  unsigned long int boundary = sqrt (x);
	  unsigned long int sqrt_x = boundary;
	  size_t n = 0;
	  size_t i = 0;
	  unsigned long int p = primes[i++];
	  size_t t;
	  size_t k;
	  size_t m = 0;
	  while (x > ULONG_MAX && p <= sqrt_x)
	    {
	      if ((x % p) == 0)
		{
		  unsigned long long int f = 1;
		  do
		    {
		      f *= p;
		      x /= p;
		    }
		  while ((x % p) == 0);
		  sqrt_x = sqrt (x);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  unsigned long int z = x;
	  while (p <= sqrt_x)
	    {
	      if ((z % p) == 0)
		{
		  unsigned long int f = 1;
		  do
		    {
		      f *= p;
		      z /= p;
		    }
		  while ((z % p) == 0);
		  sqrt_x = sqrt (z);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  if (z != 1)
	    {
	      if (z <= boundary)
		{
		  if (n == ppf_size)
		    {
		      const size_t new_size = ppf_size + 1;
		      unsigned long int *const new =
			realloc (ppf, new_size * sizeof (*ppf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  ppf_size = new_size;
			  ppf = new;
			}
		    }
		  ppf[n++] = z;
		}
	    }
	  if (n > 1)
	    {
	      qsort (ppf, n, sizeof (*ppf), cmplu);
	    }
	  t = 1 << n;
	  for (k = 0; k < t; k++)
	    {
	      size_t j;
	      unsigned long long int f = 1;
	      for (i = 0, j = 1; i < n; i++, j <<= 1)
		{
		  if ((k & j) != 0)
		    {
		      f *= ppf[i];
		    }
		}
	      if (f <= boundary)
		{
		  if (m == sf_size)
		    {
		      const size_t new_size = sf_size + 1;
		      unsigned long int *const new =
			realloc (sf, (m + 1) * sizeof (*sf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  sf_size = new_size;
			  sf = new;
			}
		    }
		  sf[m++] = f;
		}
	    }
	  if (m > 1)
	    {
	      qsort (sf, m, sizeof (*sf), cmplu);
	    }
	  for (i = 0; i < m; i++)
	    {
	      unsigned long long int f = sf[m - 1 - i];
	      unsigned long long int g = y / f;
	      f *= f;
	      g *= g;
	      const unsigned long long int b = g - f;
	      if (b > number)
		{
		  break;
		}
	      else
		{
		  const unsigned long long int c = g + f;
		  printf (format, b, number, c);
		}
	    }
	  for (; i < m; i++)
	    {
	      unsigned long long int f = sf[m - 1 - i];
	      unsigned long long int g = y / f;
	      f *= f;
	      g *= g;
	      const unsigned long long int b = g - f;
	      const unsigned long long int c = g + f;
	      printf (format, number, b, c);
	    }
	}
    }
  return true;
}

/* Find and print primitive Pythagorean triples that start with number. */
static bool
find_triples_start (const char *const restrict program_name,
		    const size_t line, const unsigned long long int number,
		    const unsigned long long int max_even,
		    const unsigned long long int max_odd,
		    size_t *restrict primes_sizep,
		    unsigned long int **const restrict primesp,
		    const char *const restrict format)
{
  size_t primes_size = *primes_sizep;
  unsigned long int *primes = *primesp;
  static const double factor = 0.643594252905582624;
  static size_t ppf_size = 0;
  static unsigned long int *ppf = NULL;
  static size_t sf_size = 0;
  static unsigned long int *sf = NULL;
  if ((number % 2) == 1)
    {
      if (number > max_odd)
	{
	  fprintf (stderr, "%s: ", program_name);
	  if (line != 0)
	    {
	      fprintf (stderr, "%zu: ", line);
	    }
	  fprintf (stderr,
		   "%llu: integer gives triple with integer overflow\n",
		   number);
	  return false;
	}
      else if (number != 1)
	{
	  unsigned long long int x = number;
	  const double sqrt_x_float = sqrt (x);
	  const unsigned long int boundary = sqrt_x_float * factor;
	  unsigned long int sqrt_x = sqrt_x_float;
	  size_t n = 0;
	  size_t i = 1;
	  unsigned long int p = primes[i++];
	  size_t t;
	  size_t k;
	  size_t m = 0;
	  while (x > ULONG_MAX && p <= sqrt_x)
	    {
	      if ((x % p) == 0)
		{
		  unsigned long long int f = 1;
		  do
		    {
		      f *= p;
		      x /= p;
		    }
		  while ((x % p) == 0);
		  sqrt_x = sqrt (x);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  unsigned long int z = x;
	  while (p <= sqrt_x)
	    {
	      if ((z % p) == 0)
		{
		  unsigned long int f = 1;
		  do
		    {
		      f *= p;
		      z /= p;
		    }
		  while ((z % p) == 0);
		  sqrt_x = sqrt (z);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  if (z != 1)
	    {
	      if (z <= boundary)
		{
		  if (n == ppf_size)
		    {
		      const size_t new_size = ppf_size + 1;
		      unsigned long int *const new =
			realloc (ppf, new_size * sizeof (*ppf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  ppf_size = new_size;
			  ppf = new;
			}
		    }
		  ppf[n++] = z;
		}
	    }
	  if (n > 1)
	    {
	      qsort (ppf, n, sizeof (*ppf), cmplu);
	    }
	  t = 1 << n;
	  for (k = 0; k < t; k++)
	    {
	      size_t j;
	      unsigned long long int f = 1;
	      for (i = 0, j = 1; i < n; i++, j <<= 1)
		{
		  if ((k & j) != 0)
		    {
		      f *= ppf[i];
		    }
		}
	      if (f <= boundary)
		{
		  if (m == sf_size)
		    {
		      const size_t new_size = sf_size + 1;
		      unsigned long int *const new =
			realloc (sf, new_size * sizeof (*sf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  sf_size = new_size;
			  sf = new;
			}
		    }
		  sf[m++] = f;
		}
	    }
	  if (m > 1)
	    {
	      qsort (sf, m, sizeof (*sf), cmplu);
	    }
	  for (i = 0; i < m; i++)
	    {
	      const unsigned long long int f = sf[m - 1 - i];
	      const unsigned long long int g = number / f;
	      const unsigned long long int u = (g + f) / 2;
	      const unsigned long long int v = (g - f) / 2;
	      const unsigned long long int b = 2 * u * v;
	      const unsigned long long int c = u * u + v * v;
	      printf (format, number, b, c);
	    }
	}
    }
  else if ((number % 4) == 0)
    {
      if (number >= max_even)
	{
	  fprintf (stderr, "%s: ", program_name);
	  if (line != 0)
	    {
	      fprintf (stderr, "%zu: ", line);
	    }
	  fprintf (stderr,
		   "%llu: integer gives triple with integer overflow\n",
		   number);
	  return false;
	}
      else
	{
	  unsigned long long int x = number / 2;
	  const unsigned long long int y = x;
	  const double sqrt_x_float = sqrt (x);
	  const unsigned long int boundary = sqrt_x_float * factor;
	  unsigned long int sqrt_x = sqrt_x_float;
	  size_t n = 0;
	  size_t i = 0;
	  unsigned long int p = primes[i++];
	  size_t t;
	  size_t k;
	  size_t m = 0;
	  while (x > ULONG_MAX && p <= sqrt_x)
	    {
	      if ((x % p) == 0)
		{
		  unsigned long long int f = 1;
		  do
		    {
		      f *= p;
		      x /= p;
		    }
		  while ((x % p) == 0);
		  sqrt_x = sqrt (x);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  unsigned long int z = x;
	  while (p <= sqrt_x)
	    {
	      if ((z % p) == 0)
		{
		  unsigned long int f = 1;
		  do
		    {
		      f *= p;
		      z /= p;
		    }
		  while ((z % p) == 0);
		  sqrt_x = sqrt (z);
		  if (f <= boundary)
		    {
		      if (n == ppf_size)
			{
			  const size_t new_size = ppf_size + 1;
			  unsigned long int *const new =
			    realloc (ppf, new_size * sizeof (*ppf));
			  if (new == NULL)
			    {
			      fprintf (stderr, "%s: ", program_name);
			      if (line != 0)
				{
				  fprintf (stderr, "%zu: ", line);
				}
			      fprintf (stderr, "%llu: ", number);
			      perror (NULL);
			      return false;
			    }
			  else
			    {
			      ppf_size = new_size;
			      ppf = new;
			    }
			}
		      ppf[n++] = f;
		    }
		}
	      if (i >= primes_size)
		{
		  if (more_primes (primes_sizep, primesp, i * 1.1 + 1) ==
		      false)
		    {
		      fprintf (stderr, "%s: ", program_name);
		      if (line != 0)
			{
			  fprintf (stderr, "%zu: ", line);
			}
		      fprintf (stderr, "%llu: ", number);
		      perror (NULL);
		      return false;
		    }
		  else
		    {
		      primes_size = *primes_sizep;
		      primes = *primesp;
		    }
		}
	      p = primes[i++];
	    }
	  if (z != 1)
	    {
	      if (z <= boundary)
		{
		  if (n == ppf_size)
		    {
		      const size_t new_size = ppf_size + 1;
		      unsigned long int *const new =
			realloc (ppf, new_size * sizeof (*ppf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  ppf_size = new_size;
			  ppf = new;
			}
		    }
		  ppf[n++] = z;
		}
	    }
	  if (n > 1)
	    {
	      qsort (ppf, n, sizeof (*ppf), cmplu);
	    }
	  t = 1 << n;
	  for (k = 0; k < t; k++)
	    {
	      size_t j;
	      unsigned long long int f = 1;
	      for (i = 0, j = 1; i < n; i++, j <<= 1)
		{
		  if ((k & j) != 0)
		    {
		      f *= ppf[i];
		    }
		}
	      if (f <= boundary)
		{
		  if (m == sf_size)
		    {
		      const size_t new_size = sf_size + 1;
		      unsigned long int *const new =
			realloc (sf, (m + 1) * sizeof (*sf));
		      if (new == NULL)
			{
			  fprintf (stderr, "%s: ", program_name);
			  if (line != 0)
			    {
			      fprintf (stderr, "%zu: ", line);
			    }
			  fprintf (stderr, "%llu: ", number);
			  perror (NULL);
			  return false;
			}
		      else
			{
			  sf_size = new_size;
			  sf = new;
			}
		    }
		  sf[m++] = f;
		}
	    }
	  if (m > 1)
	    {
	      qsort (sf, m, sizeof (*sf), cmplu);
	    }
	  for (i = 0; i < m; i++)
	    {
	      unsigned long long int f = sf[m - 1 - i];
	      unsigned long long int g = y / f;
	      f *= f;
	      g *= g;
	      const unsigned long long int b = g - f;
	      const unsigned long long int c = g + f;
	      printf (format, number, b, c);
	    }
	}
    }
  return true;
}

/* Try to read a string as an integer. */
static bool
read_string (const char *const restrict program_name,
	     const char *const restrict str, const unsigned char mode,
	     const unsigned long long int max_even,
	     const unsigned long long int max_odd,
	     size_t *restrict primes_sizep,
	     unsigned long int **const restrict primesp,
	     const char *const restrict format)
{
  char *t = NULL;
  unsigned long long int value;
  errno = 0;
  value = strtoull (str, &t, 0);
  if (*t != '\0')
    {
      fprintf (stderr, "%s: invalid positive integer: \"%s\"\n", program_name,
	       str);
      return false;
    }
  else if (errno != 0)
    {
      fprintf (stderr, "%s: ", program_name);
      perror (str);
      return false;
    }
  if (mode == 0)
    {
      return find_triples_include (program_name, 0, value, max_even, max_odd,
				   primes_sizep, primesp, format);
    }
  else
    {
      return find_triples_start (program_name, 0, value, max_even, max_odd,
				 primes_sizep, primesp, format);
    }
}

/* Read a line from a file. */
static int
get_line (char **restrict linep, size_t *restrict sizep,
	  FILE * const restrict fp)
{
  char *line = *linep;
  size_t size = *sizep;
  int c;
  bool skip = false;
  size_t k = 0;
  if (line == NULL)
    {
      size = 1;
      line = malloc (size);
      if (line == NULL)
	{
	  return EOF;
	}
      else
	{
	  *linep = line;
	  *sizep = size;
	}
    }
  while ((c = fgetc (fp)) != EOF && c != '\n')
    {
      if (skip == false)
	{
	  if (c == '#')
	    {
	      skip = true;
	    }
	  else
	    {
	      if (k == size - 1)
		{
		  size = size * 1.1 + 1;
		  line = realloc (*linep, size);
		  if (line == NULL)
		    {
		      *(*linep + k) = '\0';
		      return EOF;
		    }
		  else
		    {
		      *linep = line;
		      *sizep = size;
		    }
		}
	      line[k++] = c;
	    }
	}
    }
  line[k] = '\0';
  return c;
}

/* Read integers in standard input. */
static bool
read_stdin (const char *const restrict program_name, const unsigned char mode,
	    const unsigned long long int max_even,
	    const unsigned long long int max_odd,
	    size_t *restrict primes_sizep,
	    unsigned long int **const restrict primesp,
	    const char *const restrict format)
{
  bool return_value = true;
  size_t line = 1;
  char *str = NULL;
  size_t size = 0;
  while (get_line (&str, &size, stdin) == '\n')
    {
      char *s = str;
      while (*s != '\0')
	{
	  while (*s != '\0' && (*s < '0' || *s > '9'))
	    {
	      s++;
	    }
	  if (*s != '\0')
	    {
	      char *t = NULL;
	      unsigned long long int value;
	      if (s > str && *(s - 1) == '-')
		{
		  s--;
		}
	      errno = 0;
	      value = strtoull (s, &t, 0);
	      if (errno != 0)
		{
		  fprintf (stderr, "%s: %zu: ", program_name, line);
		  perror (s);
		  return_value = false;
		}
	      else if (mode == 0
		       && find_triples_include (program_name, line, value,
						max_even, max_odd,
						primes_sizep, primesp,
						format) == false)
		{
		  return_value = false;
		}
	      else if (mode == 1
		       && find_triples_start (program_name, line, value,
					      max_even, max_odd, primes_sizep,
					      primesp, format) == false)
		{
		  return_value = false;
		}
	      s = t;
	    }
	}
      line++;
    }
  if (feof (stdin) == 0)
    {
      fprintf (stderr, "%s: %zu: ", program_name, line);
      perror (NULL);
      return_value = false;
    }
  return return_value;
}

int
main (int argc, char *argv[])
{
  const char *const program_name = argv[0];
  int return_value = EXIT_SUCCESS;
  bool end = false;
  unsigned long int op_count = 0;
  unsigned char mode = 0;
  const char *format = "%llu %llu %llu\n";
  unsigned long long int max_even = 0;
  unsigned long long int max_odd = 0;
  size_t primes_size = 0;
  unsigned long int *primes = NULL;
  struct opt
  {
    const char *const short_opt;
    const char *const long_opt;
    const char *const help;
    unsigned char len;
    const char code;
  };
  struct opt opt_array[] = {
    {"a", "first", "Find triples that have the number as the first element.",
     0, 'a'},
    {"p", "punctuate", "Output triples with punctuation.", 0, 'p'},
    {"h", "help", "Output this help and exit.", 0, 'h'},
    {"V", "version", "Output version information and exit.", 0, 'V'},
  };
  const size_t opt_count = sizeof (opt_array) / sizeof (opt_array[0]);
  unsigned int pass_total = 2;
  unsigned int pass;
  if (argc > 1)
    {
      size_t j;
      for (j = 0; j < opt_count; j++)
	{
	  if (opt_array[j].long_opt != NULL)
	    {
	      const char *const s = strchr (opt_array[j].long_opt, '=');
	      if (s != NULL)
		{
		  opt_array[j].len = s - opt_array[j].long_opt;
		}
	      else
		{
		  opt_array[j].len = strlen (opt_array[j].long_opt);
		}
	    }
	}
    }
  for (pass = 0; pass < pass_total && end == false; pass++)
    {
      bool opt_end = false;
      int i;
      for (i = 1; i < argc && end == false; i++)
	{
	  if (opt_end == false && argv[i][0] == '-' && argv[i][1] != '\0')
	    {
	      if (argv[i][1] == '-' && argv[i][2] == '\0')
		{
		  opt_end = true;
		}
	      else
		{
		  size_t opt_match_count = 0;
		  const char *s = argv[i] + 1;
		  const char *v = NULL;
		  bool more = false;
		  do
		    {
		      size_t opt_match = 0;
		      if (more == false && *s == '-')
			{
			  s++;
			  size_t j;
			  size_t len = strlen (s);
			  v = strchr (s, '=');
			  if (v != NULL)
			    {
			      len = v - s;
			      v++;
			    }
			  for (j = 0; j < opt_count; j++)
			    {
			      if (opt_array[j].long_opt != NULL)
				{
				  if (v == NULL
				      || opt_array[j].long_opt[opt_array[j].
							       len] == '=')
				    {
				      if (len == opt_array[j].len
					  && strncmp (s,
						      opt_array[j].long_opt,
						      len) == 0)
					{
					  opt_match_count = 1;
					  opt_match = j;
					  break;
					}
				      else if (len < opt_array[j].len
					       && strncmp (s,
							   opt_array[j].
							   long_opt,
							   len) == 0)
					{
					  if (opt_match_count == 0)
					    {
					      opt_match_count = 1;
					      opt_match = j;
					    }
					  else if (opt_array[j].code !=
						   opt_array[opt_match].code)
					    {
					      opt_match_count++;
					    }
					}
				    }
				}
			    }
			  if (opt_match_count == 0)
			    {
			      fprintf (stderr,
				       "%s: unrecognized option '%s'\n",
				       program_name, argv[i]);
			      return_value = EXIT_FAILURE;
			      end = true;
			    }
			  else if (opt_match_count == 1)
			    {
			      if (v == NULL)
				{
				  if (opt_array[opt_match].
				      long_opt[opt_array[opt_match].len] ==
				      '=')
				    {
				      if (i + 1 < argc)
					{
					  v = argv[++i];
					}
				      else
					{
					  fprintf (stderr,
						   "%s: option --%.*s must have an argument\n",
						   program_name,
						   opt_array[opt_match].len,
						   opt_array[opt_match].
						   long_opt);
					  return_value = EXIT_FAILURE;
					  end = true;
					}
				    }
				}
			    }
			  else
			    {
			      size_t j;
			      fprintf (stderr,
				       "%s: option '%s' is ambiguous: '--%.*s'",
				       program_name, argv[i],
				       opt_array[opt_match].len,
				       opt_array[opt_match].long_opt);
			      for (j = opt_match + 1; j < opt_count; j++)
				{
				  if (opt_array[j].long_opt != NULL)
				    {
				      if (v == NULL
					  || opt_array[j].
					  long_opt[opt_array[j].len] == '=')
					{
					  if (len < opt_array[j].len
					      && strncmp (s,
							  opt_array[j].
							  long_opt, len) == 0)
					    {
					      fprintf (stderr, ", '--%.*s'",
						       opt_array[j].len,
						       opt_array[j].long_opt);
					    }
					}
				    }
				}
			      fputc ('\n', stderr);
			      return_value = EXIT_FAILURE;
			      end = true;
			    }
			}
		      if (end == false)
			{
			  if (opt_match_count == 0)
			    {
			      const unsigned c = *s++;
			      size_t j;
			      more = false;
			      for (j = 0; j < opt_count; j++)
				{
				  if (opt_array[j].short_opt != NULL)
				    {
				      if (c == opt_array[j].short_opt[0])
					{
					  opt_match = j;
					  if (opt_array[opt_match].
					      short_opt[1] == ' ')
					    {
					      if (*s != '\0')
						{
						  v = s;
						  s = strchr (s, '\0');
						}
					      else if (i + 1 < argc)
						{
						  v = argv[++i];
						}
					      else
						{
						  fprintf (stderr,
							   "%s: option %c must have an argument\n",
							   program_name, c);
						  return_value = EXIT_FAILURE;
						  end = true;
						}
					    }
					  else if (*s != '\0')
					    {
					      more = true;
					    }
					  break;
					}
				    }
				}
			      if (j == opt_count)
				{
				  if (isprint (c) != 0)
				    {
				      fprintf (stderr,
					       "%s: invalid option '%c'\n",
					       program_name, c);
				    }
				  else
				    {
				      fprintf (stderr,
					       "%s: invalid option '\\x%02X'\n",
					       program_name, c);
				    }
				  return_value = EXIT_FAILURE;
				  end = true;
				}
			    }
			  if (end == false)
			    {
			      const unsigned char c =
				opt_array[opt_match].code;
			      if (pass == 0)
				{
				  if (c == 'h')
				    {
				      size_t j;
				      printf
					("Usage: %s [OPTIONS] [NUMBERS]\n",
					 program_name);
				      puts
					("For each positive integer NUMBER, find primitive Pythagorean triples that include NUMBER.\n");
				      puts
					("With no NUMBER arguments, read NUMBERS in standard input.\n");
				      for (j = 0; j < opt_count; j++)
					{
					  if (opt_array[j].short_opt != NULL
					      && opt_array[j].long_opt !=
					      NULL)
					    {
					      printf ("-%s, --%s\n",
						      opt_array[j].short_opt,
						      opt_array[j].long_opt);
					    }
					  else if (opt_array[j].short_opt !=
						   NULL)
					    {
					      printf ("-%s\n",
						      opt_array[j].short_opt);
					    }
					  else if (opt_array[j].long_opt !=
						   NULL)
					    {
					      printf ("--%s\n",
						      opt_array[j].long_opt);
					    }
					  if (opt_array[j].help != NULL)
					    {
					      puts (opt_array[j].help);
					    }
					}
				      end = true;
				    }
				  else if (c == 'V')
				    {
				      puts ("triples 2.0");
				      end = true;
				    }
				  else if (c == 'L' || c == 'l')
				    {
				      op_count += 2;
				    }
				  else if (c == 'a')
				    {
				      mode = 1;
				    }
				  else if (c == 'p')
				    {
				      format = "(%llu, %llu, %llu)\n";
				    }
				}
			    }
			}
		    }
		  while (more == true && end == false);
		}
	    }
	  else if (pass == 0)
	    {
	      op_count++;
	    }
	  else
	    {
	      if (read_string
		  (program_name, argv[i], mode, max_even, max_odd,
		   &primes_size, &primes, format) == false)
		{
		  return_value = EXIT_FAILURE;
		}
	    }
	}
      if (pass == 0 && end == false)
	{
	  max_even = 2 * sqrt (ULLONG_MAX - 1) - 2;
	  max_odd = sqrt (2) * sqrt (ULLONG_MAX);
	  primes_size = 2;
	  primes = malloc (primes_size * sizeof (*primes));
	  if (primes == NULL)
	    {
	      perror (program_name);
	      return_value = EXIT_FAILURE;
	      end = true;
	    }
	  else
	    {
	      primes[0] = 2;
	      primes[1] = 3;
	      if (op_count == 0)
		{
		  pass_total = 1;
		  if (read_stdin
		      (program_name, mode, max_even, max_odd, &primes_size,
		       &primes, format) == false)
		    {
		      return_value = EXIT_FAILURE;
		    }
		}
	    }
	}
    }
  if (end == false)
    {
      if (op_count == 0)
	{
	}
    }
  if (primes != NULL)
    {
      free (NULL);
    }
  if (0 != fclose (stdout))
    {
      perror (program_name);
      return_value = EXIT_FAILURE;
    }
  return return_value;
}
